@@ -9,7 +9,7 @@ import semantic_version
 from notebook.base.handlers import IPythonHandler
 from notebook.utils import url_path_join
 
-from jupyter_cogram import __version__
+import jupyter_cogram
 
 debug_mode = os.environ.get("DEBUG_MODE", "false") == "true"
 
@@ -24,7 +24,7 @@ package_pypi_url = "https://pypi.org/pypi/jupyter-cogram/json"
 
 check_token_endpoint = "checkToken"
 
-logger.debug(f"Running jupyter-cogram version {__version__}")
+logger.debug(f"Running jupyter-cogram version {jupyter_cogram.__version__}")
 logger.debug(f"Backend URL: {backend_url}")
 logger.debug(f"Suggestions endpoint: {completions_enpdoint}")
 logger.debug(f"PyPI URL: {package_pypi_url}")
@@ -90,10 +90,10 @@ def is_installation_up_to_date(
 
     try:
         is_up_to_date = semantic_version.Version(
-            __version__
+            jupyter_cogram.__version__
         ) >= semantic_version.Version(LATEST_PYPI_VERSION)
         logger.info(
-            f"Have package version {__version__}. "
+            f"Have package version {jupyter_cogram.__version__}. "
             f"Latest PyPi version is {LATEST_PYPI_VERSION}. Installed version is up to "
             f"date: {is_up_to_date}."
         )
@@ -250,17 +250,18 @@ class JupyterCogramVersionHandler(IPythonHandler):
     def get(self):
         logger.debug(f"Checking package version")
         is_up_to_date = is_installation_up_to_date()
-
         if is_up_to_date:
             status = 200
             message = (
-                f"Installed version ({__version__}) is up to date with PyPI"
+                f"Installed version ({jupyter_cogram.__version__}) is up "
+                f"to date with PyPI"
                 f" version ({LATEST_PYPI_VERSION})."
             )
         else:
             status = 400
             message = (
-                f"Installed version ({__version__}) is **not** up to date with PyPI "
+                f"Installed version ({jupyter_cogram.__version__}) is **not** up "
+                f"to date with PyPI "
                 f"version ({LATEST_PYPI_VERSION})."
             )
 
@@ -269,12 +270,59 @@ class JupyterCogramVersionHandler(IPythonHandler):
             "status": status,
             "message": message,
             "pypi_version": LATEST_PYPI_VERSION,
-            "installed_version": __version__,
+            "installed_version": jupyter_cogram.__version__,
         }
         logger.debug(f"Returning response dict {json.dumps(response, indent=2)}")
         self.finish(json.dumps(response))
 
 
+class JupyterUpgradeCogramHandler(IPythonHandler):
+    def __init__(self, application, request, **kwargs):
+        super(JupyterUpgradeCogramHandler, self).__init__(
+            application, request, **kwargs
+        )
+
+    def post(self):
+        logger.debug(f"Receiving request to upgrade")
+        import sys
+
+        previous_version = jupyter_cogram.__version__
+        logger.debug(f"Before upgrade, have jupyter-cogram version {previous_version}")
+        cmd = f"{sys.executable} -m pip install -U jupyter-cogram"
+        logger.debug(f"Running cmd {cmd}")
+        status_code = os.system(cmd)
+
+        from importlib import reload
+
+        reload(jupyter_cogram)
+
+        new_version = jupyter_cogram.__version__
+        logger.debug(f"After upgrade, have jupyter-cogram version {new_version}")
+
+        if status_code == 0 and semantic_version.Version(
+                new_version
+        ) >= semantic_version.Version(previous_version):
+            status = 200
+            response = {
+                "status": "success",
+                "message": "Library updated.",
+                "new_version": new_version,
+            }
+        else:
+            status = 400
+            response = {
+                "status": "error",
+                "message": "Could not upgrade library. Please check your "
+                           "Jupyter Notebook server logs. If you keep encountering "
+                           "the same problem, please reach out at support@cogram.ai",
+                "new_version": LATEST_PYPI_VERSION,
+            }
+
+        self.set_status(status)
+        logger.debug(f"Returning response dict {json.dumps(response, indent=2)}")
+        self.finish(json.dumps(response))
+
+
 def load_jupyter_server_extension(nb_server_app):
     """
     Called when the extension is loaded.
@@ -293,6 +341,7 @@ def load_jupyter_server_extension(nb_server_app):
             ("/cogram", JupyterCogramHandler),
             ("/token", JupyterCogramTokenHandler),
             ("/checkVersion", JupyterCogramVersionHandler),
+            ("/upgrade", JupyterUpgradeCogramHandler),
         ]
         ],
     )
